<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use App\Helpers\Helper;
use App\Models\Booking;
use App\Models\Activity;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\URL;
use App\Events\PostActionNotification;
use App\Http\Resources\ActivityResource;
use Yajra\DataTables\Facades\DataTables;
use App\Http\Resources\PaginationCollection;

use function PHPUnit\Framework\returnSelf;

class ActivityController extends Controller
{
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = Activity::where('user_id', auth()->user()->id)
                ->with(['actor' => function ($q) {
                    $q->select('id', 'user_name');
                }])
                ->latest()
                ->get();

            // Encode the 'object_type' attribute into JSON format for each item in the $data collection
            $data = $data->map(function ($item) {
                switch ($item->object_type) {
                    case 'App\Models\PollAnswer':
                        $item->object_type = 'PollAnswer';
                        break;
                    case 'App\Models\NewsFeed':
                        $item->object_type = 'NewsFeed';
                        break;
                    case 'App\Models\Suggestion':
                        $item->object_type = 'Suggestion';
                        break;
                    case 'App\Models\Advertisement':
                        $item->object_type = 'Advertisement';
                        break;
                    case 'App\Models\Incident':
                        $item->object_type = 'Incident';
                        break;
                    case 'App\Models\FeedBack':
                        $item->object_type = 'FeedBack';
                        break;
                    case 'App\Models\ContactUs':
                        $item->object_type = 'ContactUs';
                        break;
                }
                return $item;
            });
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('created_at', function ($row) {
                    return $row->created_at->toDateString(); // Format date as needed
                })
                ->rawColumns(['created_at'])
                ->make(true);
        }
        return view('activity.Activityindex');
    }


    public function redirect(Request $request, $object_type, $object_id, $actor_id)
    {
        // Switch case to handle different object types
        switch ($object_type) {
            case 'PollAnswer':
                Activity::where('user_id', auth()->user()->id)
                    ->where('object_type', 'App\Models\PollAnswer')
                    ->where('object_id', $object_id)
                    ->where('actor_id', $actor_id)->update(['viewed' => true]);
                return redirect(route('pollcompare.view', $object_id));
                break;
            case 'Incident':
                Activity::where('user_id', auth()->user()->id)
                    ->where('object_type', 'App\Models\Incident')
                    ->where('object_id', $object_id)
                    ->where('actor_id', $actor_id)->update(['viewed' => true]);
                return redirect(route('incident.index'));
                break;
            case 'NewsFeed':
                Activity::where('user_id', auth()->user()->id)
                    ->where('object_type', 'App\Models\NewsFeed')
                    ->where('object_id', $object_id)
                    ->where('actor_id', $actor_id)->update(['viewed' => true]);
                return  redirect(route('newsfeed.show', $object_id));
                break;
            case 'Suggestion':
                Activity::where('user_id', auth()->user()->id)
                    ->where('object_type', 'App\Models\Suggestion')
                    ->where('object_id', $object_id)
                    ->where('actor_id', $actor_id)->update(['viewed' => true]);
                return redirect(route('suggestion.index'));
                break;
            case 'FeedBack':
                Activity::where('user_id', auth()->user()->id)
                    ->where('object_type', 'App\Models\FeedBack')
                    ->where('object_id', $object_id)
                    ->where('actor_id', $actor_id)->update(['viewed' => true]);
                return redirect(route('feedback.index'));
                break;
            case 'ContactUs':
                Activity::where('user_id', auth()->user()->id)
                    ->where('object_type', 'App\Models\ContactUs')
                    ->where('object_id', $object_id)
                    ->where('actor_id', $actor_id)->update(['viewed' => true]);
                return redirect(route('contactus.index'));
                break;
            case 'Advertisement':
                return redirect(route('activity.list'))->with('error', 'cant redirect this time');
            default:
                return redirect(route('activity.list'))->with('error', 'cant redirect this time');
                break;
        }
    }



    function countNotification()
    {
        $count = Activity::where('user_id', auth()->user()->id)
            ->where('viewed', 0)
            ->count();
        return response($count, 200);
    }
}
