<?php

namespace App\Http\Controllers;

use App\Events\SendAction;
use App\Http\Controllers\Controller;
use App\Models\Announcement;
use App\Models\AnnouncementMedia;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Routing\Route;

class AnnouncementController extends Controller
{
    /**
     * index
     *
     * @return void
     */
    public function index()
    {
        $announcements = Announcement::orderBy('created_at', 'desc')->get();
        return view('announcement.AnnouncmenntList', compact('announcements'));
    }

    /**
     * create
     *
     * @return void
     */
    function create()
    {
        return view('announcement.form');
    }

    function annoucnmentAdd(Request $request)
    {
        $this->validate($request, [
            // 'title' => 'required',
            'description' => 'required||max:500'
        ]);
        $announcement =  Announcement::create([
            'title' => $request->title,
            'description' => $request->description,
            // 'url' => $request->url,
        ]);
        if ($request->hasFile("media")) {
            foreach ($request->file('media') as $media) {
                $mimeType = $media->getClientMimeType();
                $before = explode('/', $mimeType);
                // return $before[0];
                AnnouncementMedia::create([
                    'announcement_id' => $announcement->id,
                    'type' => $before[0],
                    'media' => $this->multiMedia($media, "Announcment"),
                ]);
            }
        }
        // send notification
        // $media =  AnnouncementMedia::where('announcement_id', $announcement->id)->get();
        $user = auth()->user()->id;
        $users = User::where('company_id', auth()->user()->company_id)
            ->where('user_role', 'user')
            ->get(['id'])
            ->pluck('id');
        // return $user;
        event(new SendAction(
            $users->toArray(),
            [
                'title' => 'Announcement',
                'body' => $request->description,
                'object_id' => $announcement->id,
                'object_type' => get_class($announcement),
                'actor_id' => $user,
                'type' => 'announcement',
                'actor_type' => auth()->user()->user_role,
                // 'user_id' => $suggestion->user_id
            ],
            true
        ));

        return redirect(Route('announcement.list'))->with('message', 'Announcment sent Successfully');
    }
    /**
     * store
     *
     * @param  mixed $request
     * @return void
     */
    public function store(Request $request)
    {
        // return $request;
        $request->validate([
            "notification" => 'required|max:500'
        ]);
        $user = auth()->user()->id;

        $users = User::get(['id'])->pluck('id');

        event(new SendAction($users->toArray(), [
            'actor_id' => $user,
            'title' => config('constants.ANNOUNCEMENT'),
            'object' => 'announcement',
            'type' => 'announcement',
            // 'object_id' => '',
            'body' => $request->notification
        ]));

        return redirect()->back()->withSuccess('Announcement sent successfully');
    }

    function announcmentDelete(Announcement $announcement)
    {
        $announcement->delete();
        return redirect(Route('announcement.list'))->with('message', 'Announcment deleted Successfully');
    }

    // function announcmentEdit(Announcement $announcement)
    // {
    //     $files = AnnouncementMedia::where('announcement_id', $announcement->id)->get();
    //     return view('announcement.form', compact('files', 'announcement'));
    // }

    // function announcementUpdate(Request $request)
    // {
    //     return $request;
    // }
}
