<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Models\Company;
use App\Models\Industry;
use App\Models\Department;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

use Spatie\Permission\Models\Role;
use Illuminate\Support\Facades\Hash;
use Spatie\Permission\Models\Permission;
use Yajra\DataTables\Facades\DataTables;
use function PHPUnit\Framework\returnSelf;
use Illuminate\Auth\Passwords\PasswordBroker;

class CompanyController extends Controller
{
    function index(Request $request)
    {
        if ($request->ajax()) {
            $companies = User::with('company')
                ->where('user_role', 'sub_admin')
                ->latest()
                ->get();

            return DataTables::of($companies)
                ->addIndexColumn()
                ->addColumn('action', function ($user) {
                    $viewUrl = route('company.view', $user->id);
                    $editUrl = route('company.edit', $user->id);
                    $blockUnblockUrl = route('blockAndUnblock.user', ['user_id' => $user->id, 'status' => $user->status == 1 ? 0 : 1]);
                    $blockUnblockIcon = $user->status == 1 ? 'fa-lock' : 'fa-lock-open';
                    $blockUnblockColor = $user->status == 1 ? 'rgb(248, 52, 52)' : 'rgb(109, 214, 39)';

                    return '<a href="' . $viewUrl . '"><i title="User Info" class="far fa-eye" style="font-size:20px"></i></a>'
                        . '<span style="margin-left: 10%;"></span>
                        <a href="' . $editUrl . '"><i title="User Info" class="far fa-edit" style="font-size:20px"></i></a>'
                        . '<span style="margin-left: 10%;"></span>';
                    // return '<a href="' . $viewUrl . '"><i title="User Info" class="far fa-eye" style="font-size:20px"></i></a>'
                    //     . '<span style="margin-left: 10%;"><a href="#" onclick="swalll(\'' . $blockUnblockUrl . '\', \'Block/Unblock User\', \'Are you sure you want to ' . ($user->status == 1 ? 'unblock' : 'block') . ' this user?\', \'User ' . ($user->status == 1 ? 'unblocked' : 'blocked') . ' successfully\')">'
                    //     . '<i title="' . ($user->status == 1 ? 'Unblock' : 'Block') . ' User" class="fa ' . $blockUnblockIcon . '" style="font-size:20px;color: ' . $blockUnblockColor . '"></i></a></span>';
                })
                ->rawColumns(['action'])
                ->make(true);
        }

        return view("company.CompanyIndex");
    }

    function create()
    {
        $industries = Industry::get();
        $permission = Permission::get();
        return view('company.CompanyCreate', compact('industries', 'permission'));
    }

    function add(Request $request)
    {
        // company create
        $company_inputs['contact_name'] = $request->contactName;
        $company_inputs['title'] = $request->companyName;
        $company_inputs['address'] = $request->address;
        $company_inputs['industry_id'] = $request->industry_id;
        $company_inputs['registration'] = $request->registrationNumber;
        $company_inputs['website'] = $request->website;
        $company_inputs['employees'] = $request->totalEmployees;
        $company_inputs['poll_limit'] = $request->poll_limit;
        $company_inputs['invoice_contact'] = $request->invoice_contact;
        $company_inputs['invoice_Email_address'] = $request->invoice_Email_address;
        $company_inputs['company_code'] = date('YmdHis');
        $company = Company::create($company_inputs);
        //   user create
        $paswrod = $request->registrationNumber ?? '$request->registrationNumber';
        $input['password'] = Hash::make($paswrod);
        $input['user_type'] = "sub_admin";
        $input['email'] = $request->contactEmail;
        $input['phone'] = $request->contactTelephone;
        $input['company_id'] = $company->id;
        $input['user_role'] = "sub_admin";
        $input['last_name'] = $request->contactName;
        $input['first_name'] = $request->companyName;
        if ($request->hasFile('media')) {
            $input['profile_image'] = $this->media($request, 'company');
        }
        $user = User::create($input);
        // department create
        foreach ($request->department as $department) {
            Department::create(['company_id' => $company->id, 'name' => $department]);
        }
        $token = app(PasswordBroker::class)->createToken($user);
        $sql = "INSERT INTO password_resets (email, token, created_at) VALUES (?, ?, NOW())";
        DB::statement($sql, [$request->contactEmail, $token]);
        $user->update(["forgot_token" => $token]);
        $user = $user->fresh();
        // make paremission
        $role = Role::create(['name' => $request->contactEmail]);
        $input = $request->input('permission');
        $permissions = array_map('intval', $input);
        $role->syncPermissions($permissions);
        $user->assignRole($role->name);
        // send mail
        \Mail::send('auth.change-pasword-new-user', ['user' => $user], function ($m) use ($user) {
            $m->to($user->email, $user->full_name)
                ->from('support@bizzfeed.com')->subject('Account Activation Mail');
        });
        return redirect(route('company.index'))->with('status', 'company created successfully & reset email sent');
    }

    public function checkEmail(Request $request)
    {
        $email = $request->input('email');
        $existingUser = User::where('email', $email)
            ->first();
        if ($existingUser) {
            return response()->json(['available' => false]);
        }
        return response()->json(['available' => true]);
    }

    public function view($userId)
    {
        $user = User::with('company.departments', 'company.indust', 'rolesCustomRelation.permissions1')->where('id', $userId)->first();
        return view('company.CompanyInfo', compact('user'));
    }
    public function edit($userId)
    {
        $permission = Permission::get();
        $industries = Industry::get();
        $user = User::with('company.departments', 'company.industry', 'rolesCustomRelation.permissions1')->where('id', $userId)->first();
        return view('company.CompanyCreate', compact('permission', 'user', 'industries'));
    }

    function update(Request $request, $user_id)
    {
        $user = User::with('company.departments')->where('id', $user_id)->first();
        $company_inputs['title'] = $request->companyName;
        $company_inputs['industry_id'] = $request->industry_id;
        $company_inputs['address'] = $request->address;
        $company_inputs['website'] = $request->website;
        $company_inputs['contact_name'] = $request->contactName;
        $company_inputs['employees'] = $request->totalEmployees;
        $company_inputs['poll_limit'] = $request->poll_limit;
        $company_inputs['invoice_contact'] = $request->invoice_contact;
        $company_inputs['invoice_Email_address'] = $request->invoice_Email_address;
        $company = Company::where('id', $user->company_id)->update($company_inputs);
        //   user update
        $input['phone'] = $request->contactTelephone;
        $input['last_name'] = $request->contactName;
        $input['first_name'] = $request->companyName;
        if ($request->hasFile('media')) {
            $input['profile_image'] = $this->media($request, 'company');
        }
        $user->update($input);
        // update permissions
        $input = $request->input('permission');
        $permissions = array_map('intval', $input);
        if ($request->role_id != null) {
            $role = Role::find($request->role_id);
            $role->syncPermissions($permissions);
        } else {
            $role = Role::create(['name' => $request->contactEmail]);
            $role->syncPermissions($permissions);
            $user->assignRole($role->name);
        }
        // department create
        if ($request->department) {
            foreach ($request->department as $department) {
                Department::create(['company_id' => $user->company_id, 'name' => $department]);
            }
        }
        return redirect(route('company.index'))->with('message', 'company updated successfully');
    }
}
