<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use App\Models\Poll;
use App\Models\Post;
use App\Models\User;
use App\Models\Story;
use App\Models\Banner;
use App\Models\Report;
use App\Models\Company;
use App\Models\Package;
use App\Models\NewsFeed;
use App\Models\Highlight;
use App\Models\Department;
use App\Models\AppFeedback;
use Illuminate\Http\Request;

class HomeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        $countData = [];
        if (auth()->user()->user_role == 'sub_admin') {
            $companyId = auth()->user()->company_id;
            $countData["user"] = User::where('company_id', $companyId)
                ->where('user_type', "user")
                ->count();
            $countData["department"] = Department::where('company_id', $companyId)->count();
            $countData["newsfeed"] = NewsFeed::where('company_id', $companyId)->count();
            // adoptionRate
            $total_employees = Company::where('id', $companyId)->value('employees');
            if ($total_employees == 0) {
                $countData['adoption_rate'] = 'N/A';
            } else {
                $adoptionRate = $countData['user'] / $total_employees * 100;
                $countData["adoption_rate"] = number_format($adoptionRate, 1) . '%';
            }
            // retention rate
            $currentDate = Carbon::now()->toDateString();
            $firstDayOfMonth = Carbon::now()->startOfMonth()->toDateString();
            $employees_start = User::where('company_id', $companyId)
                ->where('user_role', 'user')
                ->whereDate('created_at', '<=', $firstDayOfMonth)->count();
            $employees_end = User::where('company_id', $companyId)
                ->where('user_role', 'user')
                ->whereDate('created_at', '<=', $currentDate)->count();
            if ($employees_start == 0) {
                $countData['retention_rate'] = 'N/A';
            } else {
                $countData['retention_rate'] = number_format($employees_end / $employees_start * 100, 1) . '%';
            }
            // Log in Rate
            $users_accessing_app = User::where('company_id', $companyId)
                ->where('user_role', 'user')
                ->whereNotNull('login_at')
                ->count();
            $enrolled_users   = User::where('company_id', $companyId)
                ->where('user_role', 'user')
                ->count();
            if ($enrolled_users == 0) {
                $countData['login_rate'] = 'N/A';
            } else {
                $countData['login_rate'] = number_format($users_accessing_app / $enrolled_users * 100, 1) . '%';
            }
        } else {
            $countData["company"] = User::where('user_role', "sub_admin")
                ->count();
            $countData["feedback"] = AppFeedback::count();
            $countData["stockpoll"] = Poll::where('is_stock', true)->count();
        }
        return view('home', compact('countData'));
    }
}
