<?php

namespace App\Http\Controllers;

use App\Models\Incident;
use App\Models\Department;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\URL;
use Yajra\DataTables\Facades\DataTables;
use Carbon\Carbon;


class IncidentController extends Controller
{
    function index(Request $request)
    {
        // return $incidents =  Incident::with(['user' => function ($q) {
        //     $q->select('id', 'first_name', 'last_name', 'user_name', 'department_id');
        // }])->where('company_id', auth()->user()->company_id)->get();
        // return view('incident.IncidentIndex', compact('incidents'));
        if ($request->ajax()) {
            $data =  Incident::with(['user' => function ($q) {
                $q->select('id', 'first_name', 'last_name', 'user_name', 'department_id');
            }])->where('company_id', auth()->user()->company_id)->latest()->get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($data) {
                    return '<button data-bs-toggle="modal" data-bs-target="#responsive-modal" title="Update tatus" onclick="populateModal(' . $data->id . ', \'' . $data->status . '\')" class="btn btn-info"><i class="fa fa-edit"></i></button>';
                })
                ->addColumn('media', function ($data) {
                    $mediaUrl = $data->media != null ? env('MEDIA_URL') . $data->media : 'images/placeholderprofile.png';

                    // Get the file extension from the media URL
                    $extension = pathinfo($mediaUrl, PATHINFO_EXTENSION);

                    if (in_array($extension, ['jpg', 'jpeg', 'png', 'gif'])) {
                        return '<a href="' . URL::asset($mediaUrl) . '" target="_blank"><img src="' . URL::asset($mediaUrl) . '" class="img-circle" style="height: 50px; width: 50px;" /></a>';
                    } elseif (in_array($extension, ['mp4', 'webm', 'ogg'])) {
                        return '<a href="' . URL::asset($mediaUrl) . '" target="_blank"><video src="' . URL::asset($mediaUrl) . '" controls width="100">Your browser does not support the video tag.</video></a>';
                    } else {
                        return 'Unsupported media type';
                    }
                })
                ->addColumn('created_at', function ($data) {
                    return $data->created_at->format('Y-m-d');
                })
                ->rawColumns(['action', 'media'])
                ->make(true);
        }
        $departments = Department::where('company_id', auth()->user()->company_id)->get();
        return view('incident.IncidentIndex', compact('departments'));
    }

    function incidentStatus(Request $request)
    {
        $updateData = ['status' => $request->status, 'resolved_at' => null];
        if ($request->status === 'resolved') {
            $updateData['resolved_at'] = now();
        }
        Incident::where('id', $request->incidentId)->update($updateData);
        return redirect()->back()->with('message', 'status updated successfully');
    }


    public function summaryByDayAndStatus()
    {
        $currentYear = Carbon::now()->year;
        $companyID = auth()->user()->company_id;
        $avgHoursToRepair = DB::table('incidents')
            ->selectRaw('COUNT(*) as total_incidents, SUM(TIMESTAMPDIFF(SECOND, created_at, resolved_at)) / COUNT(*) / 3600 AS avg_hours_to_repair')
            ->where('company_id', $companyID)
            ->where('status', 'resolved')
            ->first();

        $departments = Department::where('company_id', $companyID)->get();

        // Fetch incidents with necessary aggregation by month
        $incidents = DB::table('incidents')
            ->select(
                DB::raw('DATE_FORMAT(created_at, "%M") as month'),
                'status',
                DB::raw('count(*) as total')
            )
            ->where('company_id', $companyID)
            ->whereYear('created_at', $currentYear)
            ->groupBy(DB::raw('DATE_FORMAT(created_at, "%M")'), 'status')
            ->get();

        // Format data for Chart.js
        $formattedData = [];
        foreach ($incidents as $incident) {
            $month = $incident->month;
            $status = $incident->status;
            $total = $incident->total;

            if (!isset($formattedData[$month])) {
                $formattedData[$month] = ['month' => $month, 'statuses' => []];
            }
            $formattedData[$month]['statuses'][$status] = $total;
        }

        return view('incident.IncidentSummaries', compact('avgHoursToRepair', 'formattedData', 'departments'));
    }

    public function summaryByDayAndStatusAjax(Request $request)
    {
        $currentYear = Carbon::now()->year;
        $companyID = auth()->user()->company_id;

        $month = $request->input('month');
        $departmentID = $request->input('department_id');

        $query = DB::table('incidents')
            ->join('users', 'incidents.user_id', '=', 'users.id')
            ->where('incidents.company_id', $companyID)
            ->whereYear('incidents.created_at', $currentYear);

        if ($month) {
            $query->whereMonth('incidents.created_at', $month);
        }

        if ($departmentID) {
            $query->where('users.department_id', $departmentID);
        }

        if (!$month) {
            // Group by month if no specific month is selected
            $query->select(
                DB::raw('DATE_FORMAT(incidents.created_at, "%M") as month'),
                'incidents.status',
                DB::raw('count(*) as total')
            )->groupBy(DB::raw('DATE_FORMAT(incidents.created_at, "%M")'), 'incidents.status');
        } else {
            // Group by day if a specific month is selected
            $query->select(
                DB::raw('DATE(incidents.created_at) as date'),
                'incidents.status',
                DB::raw('count(*) as total')
            )->groupBy(DB::raw('DATE(incidents.created_at)'), 'incidents.status');
        }

        $incidents = $query->get();

        $formattedData = [];
        foreach ($incidents as $incident) {
            $date = $month ? $incident->date : $incident->month;
            $status = $incident->status;
            $total = $incident->total;

            if (!isset($formattedData[$date])) {
                $formattedData[$date] = ['date' => $date, 'statuses' => []];
            }
            $formattedData[$date]['statuses'][$status] = $total;
        }

        return response()->json($formattedData);
    }
}
