<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use App\Models\User;
use App\Models\NewsFeed;
use App\Events\SendAction;
use App\Models\Department;
use Illuminate\Http\Request;
use App\Models\NewsFeedsMedia;
use Illuminate\Support\Facades\Auth;
use PhpParser\Node\Stmt\Return_;

class NewsFeedController extends Controller
{

    public function index()
    {
        $companyId = auth()->user()->company_id;
        $newsfeeds = NewsFeed::with('media', 'comments.user') // Eager load the media relationship
            ->withCount(['reach', 'likes', 'comments']) // Get the counts of reach and likes
            ->where('company_id', $companyId)
            ->latest()->get();

        return view('NewsFeed.NewsFeedList', compact('newsfeeds'));
    }

    function show($newsfeed)
    {
        $news_feed = NewsFeed::with('media', 'comments.user') // Eager load the media relationship
            ->withCount(['reach', 'likes', 'comments']) // Get the counts of reach and likes
            ->where('id', $newsfeed)
            ->first();
        return view('NewsFeed.NewsFeedDeatil', compact('news_feed'));
    }

    function create()
    {
        return view('NewsFeed.NewsFeedCreate');
    }

    function add(Request $request)
    {
        $validateData = $request->validate([
            'description' => 'required|max:255'
        ]);
        $validateData['user_id'] = auth()->user()->id;
        $validateData['company_id'] = auth()->user()->company_id;
        $newsfeed = NewsFeed::create($validateData);
        if ($request->hasFile('images')) {
            foreach ($request->file('images') as $media) {
                $mimeType = $media->getClientMimeType();
                $mediaType = explode('/', $mimeType)[0];
                $path = $this->multiMedia($media, 'product');
                $newsfeed->media()->create([
                    'media' => $path,
                    'media_type' => $mediaType,
                    'news_feed_id' => $newsfeed->id,
                ]);
            }
        }
        $user = auth()->user()->id;
        $users = User::where('company_id', auth()->user()->company_id)
            ->where('user_role', 'user')
            ->get(['id'])
            ->pluck('id');
        // return $user;
        event(new SendAction(
            $users->toArray(),
            [
                'title' => 'admin post news feed',
                'body' => $request->description,
                'object_id' => $newsfeed->id,
                'object_type' => get_class($newsfeed),
                'actor_id' => $user,
                'type' => 'create-feed',
                'actor_type' => auth()->user()->user_role,
                // 'user_id' => $suggestion->user_id
            ],
            true
        ));
        return redirect(route('newsfeed.index'))->with('message', 'news feed created successfully');
    }

    function edit($newsfeed)
    {
        $news_feed = NewsFeed::with('media', 'comments.user') // Eager load the media relationship
            ->withCount(['reach', 'likes', 'comments']) // Get the counts of reach and likes
            ->where('id', $newsfeed)
            ->first();
        return view('NewsFeed.NewsFeedCreate', compact('news_feed'));
    }

    function delete(NewsFeed $newsfeed)
    {
        $newsfeed->delete();
        return redirect(route('newsfeed.index'))->with('message', 'news feed deleted successfully');
    }

    function update(Request $request, NewsFeed $newsfeed)
    {
        $validateData = $request->validate([
            'description' => 'required|max:255'
        ]);
        if ($request->deleted_image_ids) {
            $deleted_image_ids =  json_decode('[' . $request->deleted_image_ids . ']');
            foreach ($deleted_image_ids as $image_id) {
                $image = NewsFeedsMedia::find($image_id);
                if ($image) {
                    $image->delete();
                }
            }
        }
        if ($request->hasFile('images')) {
            foreach ($request->file('images') as $media) {
                $mimeType = $media->getClientMimeType();
                $mediaType = explode('/', $mimeType)[0];
                $path = $this->multiMedia($media, 'product');
                $newsfeed->media()->create([
                    'media' => $path,
                    'media_type' => $mediaType,
                    'news_feed_id' => $newsfeed->id,
                ]);
            }
        }
        $newsfeed->update($validateData);
        return redirect(route('newsfeed.index'))->with('message', 'news feed updated successfully');
    }

    function analyticsView($newsfeed)
    {
        $auCompnaythId = auth()->user()->company_id;
        $departments = Department::where('company_id', $auCompnaythId)->get();
        $newsfeed = NewsFeed::find($newsfeed);
        $commentsUsersIds = $newsfeed->comments->pluck('user_id');
        $reactableUserIds = $newsfeed->likes->pluck('user_id');
        $reachCount = $newsfeed->reach->count();
        $uniqueUserIds = $commentsUsersIds->merge($reactableUserIds)->unique();
        $engagementCount = $uniqueUserIds->count();
        // return $engagementCount . '/' . $reachCount;
        $companyUserCount = User::where(
            [
                'company_id' => $auCompnaythId,
                'user_role' => 'user'
            ]
        )->count();
        $openRate = $companyUserCount > 0 ? round(($reachCount / $companyUserCount) * 100, 1) : 0;
        $ClickThroughRate = $reachCount > 0 ? round(($engagementCount / $reachCount) * 100, 1) : 0;
        // peaktime linechart
        $startOfDay = Carbon::today()->startOfDay();
        $endOfDay = Carbon::today()->endOfDay();
        $clicks = $newsfeed->likes;
        $opens = $newsfeed->reach;
        $peaktimeMap = $this->peaktimeData($opens, $clicks, $date = null);
        return view('NewsFeed.NewsFeedAnalytics', compact('peaktimeMap', 'newsfeed', 'ClickThroughRate', 'departments', 'openRate'));
    }

    public function getNewsfeedOpenRateDepartmentData(Request $request)
    {
        $newsfeedId = $request->input('newsfeed_id');
        $departmentId = $request->input('department_id');
        $newsFeed = NewsFeed::where('id', $newsfeedId)->first();
        if ($departmentId) {
            $departmentUsers = User::where('company_id', $newsFeed->company_id)
                ->where('department_id', $departmentId)
                ->pluck('id');
        } else {
            $departmentUsers = User::where('company_id', $newsFeed->company_id)->where('user_role', 'user')->pluck('id');
        }
        $totalReactCounts = $newsFeed->reach()
            ->whereIn('user_id', $departmentUsers)
            ->count();
        $totalDepartmentUsers = $departmentUsers->count();
        $newsfeedOpenRatePercentage = $totalDepartmentUsers > 0 ? ($totalReactCounts / $totalDepartmentUsers) * 100 : 0;
        return response()->json([
            'percentage' => round($newsfeedOpenRatePercentage, 1)
        ]);
    }

    function getNewsFeedClickThroughRateDepartmentData(Request $request)
    {
        $newsfeedId = $request->input('newsfeed_id');
        $departmentId = $request->input('department_id');
        $newsFeed = NewsFeed::find($newsfeedId);
        if ($departmentId) {
            $departmentUsers = User::where('company_id', $newsFeed->company_id)
                ->where('department_id', $departmentId)
                ->pluck('id');
        } else {
            $departmentUsers = User::where('company_id', $newsFeed->company_id)->where('user_role', 'user')->pluck('id');
        }
        $commentsUsersIds = $newsFeed->comments()->whereIn('user_id', $departmentUsers)->pluck('user_id');
        $reactableUserIds = $newsFeed->likes()->whereIn('user_id', $departmentUsers)->pluck('user_id');
        $reachCount = $newsFeed->reach->count();
        $uniqueUserIds = $commentsUsersIds->merge($reactableUserIds)->unique();
        $engagementCount = $uniqueUserIds->count();
        $ClickThroughRate = $reachCount > 0 ? round(($engagementCount / $reachCount) * 100, 1) : 0;
        return response()->json([
            'percentage' => round($ClickThroughRate, 1)
        ]);
    }
    function getbyDayChart(Request $request)
    {
        if ($request->newsfeed_id) {
            $newsfeed = NewsFeed::find($request->newsfeed_id);
            $clicks = $newsfeed->likes;
            $opens = $newsfeed->reach;
        } else {
            $auCompanyId = auth()->user()->company_id;
            // Get all news feeds for the company
            $newsFeeds = NewsFeed::where('company_id', $auCompanyId)->get();
            // Initialize counts and collections
            $totalReach = collect();
            $totalLike = collect();
            // Iterate over each news feed to sum reach and likes
            foreach ($newsFeeds as $newsFeed) {
                // for chart data
                $totalLike = $totalLike->merge($newsFeed->likes);
                $totalReach = $totalReach->merge($newsFeed->reach);
            }
            $clicks = $totalLike;
            $opens = $totalReach;
        }
        $date = $request->date;
        $peaktimeMap = $this->peaktimeData($opens, $clicks, $date);
        return response()->json([
            $peaktimeMap
        ]);
    }
    function dataOverview()
    {
        $auCompanyId = auth()->user()->company_id;
        // Get all news feeds for the company
        $newsFeeds = NewsFeed::where('company_id', $auCompanyId)->get();
        $newsFeedCount = $newsFeeds->count();
        // Initialize counts and collections
        $totalReachCount = 0;
        $totalLikeCount = 0;
        $commentsUsersIds = collect();  // Create an empty collection for reach user IDs
        $reactableUserIds = collect();  // Create an empty collection for like user IDs
        $totalReach = collect();
        $totalLike = collect();
        // Iterate over each news feed to sum reach and likes
        foreach ($newsFeeds as $newsFeed) {
            $totalReachCount += $newsFeed->reach->count();
            $totalLikeCount += $newsFeed->likes->count();
            // for chart data
            $totalLike = $totalLike->merge($newsFeed->likes);
            $totalReach = $totalReach->merge($newsFeed->reach);

            // Merge user IDs into the collections for calculation
            $commentsUsersIds = $commentsUsersIds->merge($newsFeed->comments->pluck('user_id'));
            $reactableUserIds = $reactableUserIds->merge($newsFeed->likes->pluck('user_id'));
        }
        // return $totalLike;
        // Get unique user IDs for engagement count
        $uniqueUserIds = $commentsUsersIds->merge($reactableUserIds)->unique();
        $engagementCount = $uniqueUserIds->count();
        // Count company users with 'user' role
        $companyUserCount = User::where([
            'company_id' => $auCompanyId,
            'user_role' => 'user'
        ])->count();
        // Calculate open rate and click-through rate
        $peaktimeMap = $this->peaktimeData($totalReach, $totalLike, $date = null);
        $openRate = $companyUserCount > 0 ? round(($totalReachCount / $companyUserCount) * 100, 1) : 0;
        $clickThroughRate = $totalReachCount > 0 ? round(($engagementCount / $totalReachCount) * 100, 1) : 0;
        return view('NewsFeed.NewsFeedOverview', compact('peaktimeMap', 'openRate', 'clickThroughRate', 'newsFeedCount'));
    }

    private function peaktimeData($opens, $clicks, $date)
    {
        $data = [
            '1hr' => ['labels' => [], 'open' => [], 'click' => []],
            '3hr' => ['labels' => [], 'open' => [], 'click' => []],
            '6hr' => ['labels' => [], 'open' => [], 'click' => []],
            'full' => ['labels' => [], 'open' => [], 'click' => []]
        ];

        // Define intervals
        $timeIntervals = [
            '1hr' => [
                'duration' => 1,
                'step' => 15
            ],
            '3hr' => [
                'duration' => 3,
                'step' => 30
            ],
            '6hr' => [
                'duration' => 6,
                'step' => 60
            ],
            'full' => [
                'duration' => 24,
                'step' => 60
            ]
        ];

        // Calculate base date
        $baseDate = $date ? Carbon::parse($date)->startOfDay() : Carbon::now()->startOfDay();

        foreach ($timeIntervals as $interval => $details) {
            $hours = $details['duration'];
            $stepMinutes = $details['step'];

            for ($i = 0; $i <= $hours * 60; $i += $stepMinutes) {
                $label = sprintf('%02d:%02d', floor($i / 60), $i % 60);
                $data[$interval]['labels'][] = $label;

                $start = $baseDate->copy()->addMinutes($i);
                $end = $baseDate->copy()->addMinutes($i + $stepMinutes);

                $openCount = $opens->whereBetween('created_at', [$start, $end])->count();
                $clickCount = $clicks->whereBetween('created_at', [$start, $end])->count();

                $data[$interval]['open'][] = $openCount;
                $data[$interval]['click'][] = $clickCount;
            }
        }

        return $data;
    }
}
