<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use App\Models\Poll;
use App\Models\User;
use App\Models\React;
use App\Models\Company;
use App\Models\Industry;
use App\Events\SendAction;
use App\Models\Department;
use App\Models\PollAnswer;
use Illuminate\Http\Request;
use App\Models\StockPollPrivacy;
use App\Traits\CustomAnalytics;
use Illuminate\Support\Facades\DB;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx\Rels;

class PollController extends Controller
{
    use CustomAnalytics;
    function index()
    {
        $userCount = User::where('company_id', auth()->user()->company_id)
            ->where('user_role', 'user')->count();
        $appUserCount = User::where('user_role', 'user')->count();
        // $polls = Poll::with(['answers' => function ($q) {
        //     $q->withCount('reactCounts');
        // }])
        //     ->where('company_id', auth()->user()->company_id)
        //     ->latest()
        //     ->get();
        $polls = Poll::with([
            'pollStatus',
            'answers' => function ($q) {
                $q->withCount('reactCounts');
            }
        ])
            ->when(auth()->user()->user_role == 'admin', function ($query) {
                return $query->where('is_stock', true);
            }, function ($query) {
                return $query->where('company_id', auth()->user()->company_id)->orWhere('is_stock', true);
            })
            ->latest()
            ->get();
        $is_stock = Poll::where('is_stock', true)->count();
        return view('poll.PollList', compact('polls', 'userCount', 'is_stock', 'appUserCount'));
    }
    function create()
    {
        return view('poll.PollCreateUpdate');
    }

    function add(Request $request)
    {
        // return $request;
        $validateInputs = $request->validate([
            'end_date' => 'required',
            'answer' => 'required|array',
            'question' => 'required',
        ]);
        // return $request->answer;
        if (auth()->user()->user_role == 'sub_admin') {
            $companyId = auth()->user()->company_id;
            $company = Company::where('id', $companyId)->first();
            $pollstCount = $company->polls()
                ->where('created_at', '>=', Carbon::now()->startOfMonth())
                ->count();
            if ($pollstCount >= $company->poll_limit) {
                return redirect()->back()->with(['error' => 'You have reached the limit of ' . $company->poll_limit . ' polls for this month.']);
            }
            $validateInputs['company_id'] = $companyId;
            $validateInputs['user_id'] = auth()->user()->id;
            $poll = Poll::create($validateInputs);
            foreach ($request->answer as $answer) {
                $ans = strip_tags($answer['answer']);
                $data['color'] = $answer['color'];
                $data['poll_id'] = $poll->id;
                $data['grade'] = $answer['grade'];
                $data['emoji'] = $answer['image'];
                $data['answer'] = $ans;
                PollAnswer::create($data);
            }
            $user = auth()->user()->id;
            $users = User::where('company_id', auth()->user()->company_id)
                ->where('user_role', 'user')
                ->get(['id'])
                ->pluck('id');
            // return $user;
            event(
                new SendAction(
                    $users->toArray(),
                    [
                        'title' => 'Admin created a poll',
                        'body' => 'Admin created a poll',
                        'object_id' => $poll->id,
                        'object_type' => get_class($poll),
                        'actor_id' => $user,
                        'type' => 'create-poll',
                        'actor_type' => auth()->user()->user_role,
                        // 'user_id' => $suggestion->user_id
                    ],
                    true
                )
            );
            return redirect(route('poll.index'))->with('message', 'poll created successfully');
        } else {

            $validateInputs['is_stock'] = true;
            $validateInputs['company_id'] = false;
            $validateInputs['user_id'] = auth()->user()->id;
            $poll = Poll::create($validateInputs);
            foreach ($request->answer as $answer) {
                $ans = strip_tags($answer['answer']);
                $data['color'] = $answer['color'];
                $data['poll_id'] = $poll->id;
                $data['grade'] = $answer['grade'];
                $data['emoji'] = $answer['image'];
                $data['answer'] = $ans;
                PollAnswer::create($data);
            }
            return redirect(route('poll.index'))->with('message', 'stock poll created successfully');
        }
    }

    function edit(Poll $poll)
    {
        $poll = Poll::with('answers')->find($poll->id);
        return view('poll.PollCreateUpdate', compact('poll'));
    }

    function update(Request $request, Poll $poll)
    {
        // return $request;
        if (!empty($request->deleted_able_ids)) {
            $poll->answers()->whereIn('id', $request->deleted_able_ids)->delete();
        }
        $poll->update(['question' => $request->question, 'end_date' => $request->end_date]);
        foreach ($request->answer as $key => $value) {
            $ans = strip_tags($value['answer']);
            if ($value['id'] > 0) {
                PollAnswer::where('id', $value['id'])
                    ->update([
                        'answer' => $ans,
                        'color' => $value['color'],
                        'grade' => $value['grade'],
                        'emoji' => $value['image'],
                    ]);
            } else {
                PollAnswer::create([
                    'answer' => $ans,
                    'poll_id' => $poll->id,
                    'color' => $value['color'],
                    'grade' => $value['grade'],
                    'emoji' => $value['image'],
                ]);
            }
        }
        return redirect(route('poll.index'))->with('message', 'poll updated successfully');
    }

    function delete(poll $poll)
    {
        $poll->delete();
        return redirect(route('poll.index'))->with('message', 'poll deleted successfully');
    }

    function compareView($poll_id)
    {
        if (auth()->user()->user_role == 'admin') {
            $polls = Poll::where('id', $poll_id)->with([
                'answers' => function ($q) {
                    $q->withCount('reactCounts');
                }
            ])->first();
        } else {
            $pollPrivacy = StockPollPrivacy::where('poll_id', $poll_id)->where('user_id', '!=', auth()->user()->id)->pluck('company_id')->toArray();
            $PrivacyUsers = User::whereIn('company_id', $pollPrivacy)->pluck('id')->toArray();
            $polls = Poll::where('id', $poll_id)->with([
                'answers' => function ($q) use ($PrivacyUsers) {
                    $q->withCount([
                        'reactCounts' => function ($query) use ($PrivacyUsers) {
                            $query->whereNotIn('user_id', $PrivacyUsers);
                        }
                    ]);
                }
            ])->first();
        }
        $userCount = User::where('company_id', $polls->company_id)
            ->where('user_role', 'user')->count();
        $Companies = Company::get();
        $Industries = Industry::get();
        return view(
            'poll.PollChart',
            compact(
                'Industries',
                'Companies',
                'polls',
                'userCount'
            )
        );
    }

    function compareDataLeft(Request $request)
    {
        if (auth()->user()->user_role == 'admin') {
            $userIdArray = User::where('company_id', $request->company_id)->pluck('id')->toArray();
            $polls = Poll::where('id', $request->poll_id)->with([
                'answers' => function ($q) use ($userIdArray) {
                    $q->withCount([
                        'reactCounts' => function ($r) use ($userIdArray) {
                            $r->whereIn('user_id', $userIdArray);
                        }
                    ]);
                }
            ])->first();
        } else {
            $pollPrivacy = StockPollPrivacy::where(['poll_id' => $request->poll_id, 'company_id' => $request->company_id])->where('user_id', '!=', auth()->user()->id)->pluck('company_id')->toArray();
            if ($pollPrivacy) {
                $all_users = User::pluck('id')->toArray();
                $PrivacyUsers = User::whereIn('company_id', $pollPrivacy)->pluck('id')->toArray();
                $polls = Poll::where('id', $request->poll_id)->with([
                    'answers' => function ($q) use ($all_users) {
                        $q->withCount([
                            'reactCounts' => function ($r) use ($all_users) {
                                $r->whereNotIn('user_id', $all_users);
                            }
                        ]);
                    }
                ])->first();
                // dd($PrivacyUsers);
            } else {
                $PrivacyUsers = User::where('company_id', $request->company_id)->pluck('id')->toArray();
                $polls = Poll::where('id', $request->poll_id)->with([
                    'answers' => function ($q) use ($PrivacyUsers) {
                        $q->withCount([
                            'reactCounts' => function ($r) use ($PrivacyUsers) {
                                $r->whereIn('user_id', $PrivacyUsers);
                            }
                        ]);
                    }
                ])->first();
            }
        }
        $company = Company::where('id', $request->company_id)->first('title');
        $department = Department::where('company_id', $request->company_id)->get();

        // Assigning the company title to the poll object
        $polls['CompanyTitle'] = $company->title;
        $polls['departments'] = $department;
        return response($polls, 200);
    }
    function compareDataLeftByDepartment(Request $request)
    {
        if ($request->department_id == null && auth()->user()->user_role == 'admin') {
            $userIdArray = User::where('company_id', $request->company_id)->pluck('id')->toArray();
            $polls = Poll::where('id', $request->poll_id)->with([
                'answers' => function ($q) use ($userIdArray) {
                    $q->withCount([
                        'reactCounts' => function ($r) use ($userIdArray) {
                            $r->whereIn('user_id', $userIdArray);
                        }
                    ]);
                }
            ])->first();
        } elseif ($request->department_id == null && auth()->user()->user_role == 'sub_admin') {
            $pollPrivacy = StockPollPrivacy::where(['poll_id' => $request->poll_id, 'company_id' => $request->company_id])->where('user_id', '!=', auth()->user()->id)->pluck('company_id')->toArray();
            if ($pollPrivacy) {
                $all_users = User::pluck('id')->toArray();
                $PrivacyUsers = User::whereIn('company_id', $pollPrivacy)->pluck('id')->toArray();
                $polls = Poll::where('id', $request->poll_id)->with([
                    'answers' => function ($q) use ($all_users) {
                        $q->withCount([
                            'reactCounts' => function ($r) use ($all_users) {
                                $r->whereNotIn('user_id', $all_users);
                            }
                        ]);
                    }
                ])->first();
                // dd($PrivacyUsers);
            } else {
                $PrivacyUsers = User::where('company_id', $request->company_id)->pluck('id')->toArray();
                $polls = Poll::where('id', $request->poll_id)->with([
                    'answers' => function ($q) use ($PrivacyUsers) {
                        $q->withCount([
                            'reactCounts' => function ($r) use ($PrivacyUsers) {
                                $r->whereIn('user_id', $PrivacyUsers);
                            }
                        ]);
                    }
                ])->first();
            }
        } else {
            if (auth()->user()->user_role == 'admin') {
                $userIdArray = User::where('department_id', $request->department_id)->pluck('id')->toArray();
                $polls = Poll::where('id', $request->poll_id)->with([
                    'answers' => function ($q) use ($userIdArray) {
                        $q->withCount([
                            'reactCounts' => function ($r) use ($userIdArray) {
                                $r->whereIn('user_id', $userIdArray);
                            }
                        ]);
                    }
                ])->first();
            } else {
                $pollPrivacy = StockPollPrivacy::where(['poll_id' => $request->poll_id, 'company_id' => $request->company_id])->where('user_id', '!=', auth()->user()->id)->pluck('company_id')->toArray();
                $all_users = User::pluck('id')->toArray();
                if ($pollPrivacy) {
                    $PrivacyUsers = User::whereIn('company_id', $pollPrivacy)->pluck('id')->toArray();
                    $polls = Poll::where('id', $request->poll_id)->with([
                        'answers' => function ($q) use ($all_users) {
                            $q->withCount([
                                'reactCounts' => function ($r) use ($all_users) {
                                    $r->whereNotIn('user_id', $all_users);
                                }
                            ]);
                        }
                    ])->first();
                } else {
                    $userIdArray = User::where('department_id', $request->department_id)->pluck('id')->toArray();
                    $polls = Poll::where('id', $request->poll_id)->with([
                        'answers' => function ($q) use ($userIdArray) {
                            $q->withCount([
                                'reactCounts' => function ($r) use ($userIdArray) {
                                    $r->whereIn('user_id', $userIdArray);
                                }
                            ]);
                        }
                    ])->first();
                }
            }
        }
        $company = Company::where('id', $request->company_id)->first('title');
        // Assigning the company title to the poll object
        $polls['CompanyTitle'] = $company->title;
        return response($polls, 200);
    }

    function getCompaniesByIndustry(Request $request)
    {
        $companies = Company::whereIn('industry_id', $request->industry_ids)
            ->get(['id', 'title']);

        // Return the companies as JSON response
        return response()->json($companies);
    }

    function pollType(Request $request)
    {
        // Poll::where('id', $request->id)->update(['type' => $request->type]);
        if ($request->type == 'private') {
            StockPollPrivacy::create([
                'user_id' => auth()->user()->id,
                'company_id' => auth()->user()->company_id,
                'poll_id' => $request->id
            ]);
        } else {
            StockPollPrivacy::where('poll_id', $request->id)
                ->where('user_id', auth()->user()->id)
                ->delete();
        }
        return response(['success' => true], 200);
    }

    function analyticsView(Request $request, $id)
    {
        $pollId = $id;
        $companyId = auth()->user()->company_id;
        $departments = Department::where('company_id', $companyId)->get();
        $total_company_users = User::where('company_id', $companyId)
            ->where('user_role', 'user')->count();
        $pieData = $this->customPieChart($total_company_users, $id, $companyId);
        $totalReactCountByMonth = $this->CustomLinchart($id);
        $pollAnswers = $this->customBArChart($total_company_users, $id, $companyId);
        $clickThroughRateCount = $this->clickThroughRateCount($id);
        $CommunicationsEngagementPercentage = $this->CommunicationsReachEngagement($total_company_users, $id, $companyId);
        $PeakTimesClickMaps = $this->PeakTimesClickMaps($total_company_users, $id, $companyId);
        return view(
            'poll.PollAnalytics',
            compact(
                'CommunicationsEngagementPercentage',
                'pollAnswers',
                'totalReactCountByMonth',
                'pieData',
                'departments',
                'pollId',
                'clickThroughRateCount',
                'PeakTimesClickMaps',
            )
        );
    }
    // open rate filter data
    public function getOpenRateDepartmentData(Request $request)
    {
        $pollId = $request->input('poll_id');
        $departmentId = $request->input('department_id');

        $poll = Poll::where('id', $pollId)->with([
            'answers' => function ($query) {
                $query->withCount('reactCounts');
            }
        ])->first();

        if ($departmentId) {
            $departmentUsers = User::where('company_id', $poll->company_id)
                ->where('department_id', $departmentId)
                ->pluck('id');
        } else {
            $departmentUsers = User::where('company_id', $poll->company_id)->where('user_role', 'user')->pluck('id');
        }
        $totalReactCounts = $poll->answers->sum(function ($answer) use ($departmentUsers) {
            return $answer->reactCounts()->whereIn('user_id', $departmentUsers)->count();
        });
        $totalDepartmentUsers = $departmentUsers->count();
        $engagementPercentage = $totalDepartmentUsers > 0 ? ($totalReactCounts / $totalDepartmentUsers) * 100 : 0;
        return response()->json([
            'percentage' => round($engagementPercentage, 1)
        ]);
    }
    public function getClickThroughRateDepartmentData(Request $request)
    {
        $pollId = $request->input('poll_id');
        $departmentId = $request->input('department_id');

        $poll = Poll::where('id', $pollId)->with([
            'answers' => function ($query) {
                $query->withCount('reactCounts');
            }
        ])->first();

        if ($departmentId) {
            $departmentUsers = User::where('company_id', $poll->company_id)
                ->where('department_id', $departmentId)
                ->pluck('id');
        } else {
            $departmentUsers = User::where('company_id', $poll->company_id)->where('user_role', 'user')->pluck('id');
        }
        // Sum up the react_counts_count from answers specific to department users
        $totalReactCounts = $poll->answers->sum(function ($answer) use ($departmentUsers) {
            return $answer->reactCounts()->whereIn('user_id', $departmentUsers)->count();
        });
        // Calculate total open specific to department users
        $totalOpen = Poll::where('id', $pollId)
            ->withCount([
                'clickThroughRate' => function ($query) use ($departmentUsers) {
                    $query->whereIn('user_id', $departmentUsers);
                }
            ])
            ->first()
            ->click_through_rate_count;
        $clickOpenRatePercentage = $totalOpen > 0 ? ($totalReactCounts / $totalOpen) * 100 : 0;
        return response()->json([
            'percentage' => round($clickOpenRatePercentage, 1),
        ]);
    }
    public function dataOverview($id)
    {
        $poll = Poll::with(['clickThroughRate', 'answers.reactCounts'])->findOrFail($id);
        $totalOpen = $poll->clickThroughRate->count(); // Assuming clickThroughRate is a relationship
        $answers = $poll->answers;
        $openResponses = $poll->clickThroughRate;
        $totalCompany = User::where('user_role', 'sub_admin')->count();
        $lastTwoMonthsData = $this->getCurrentAndPreviousMonthData($answers, $openResponses);
        $openRate = $this->dataOverviewOpenRate($id);
        $clickThroughRate = $this->clickThroughRateCount($id);
        return view('poll.DataOverview', compact('openRate', 'lastTwoMonthsData', 'totalCompany', 'clickThroughRate'));
    }
    private function getCurrentAndPreviousMonthData($answers, $openResponses)
    {
        $data = [
            'labels' => [],
            'open' => [],
            'click' => [],
        ];
        $currentYear = Carbon::now()->year;
        $months = [
            Carbon::now()->subMonth()->format('F') => Carbon::now()->subMonth()->format('M'),
            Carbon::now()->format('F') => Carbon::now()->format('M'),
        ];
        // Loop through the current and previous months
        foreach ($months as $fullMonthName => $shortMonthName) {
            $weekDataOpen = [];
            $weekDataClick = [];
            // Get the first and last days of the month
            $firstDayOfMonth = Carbon::parse("first day of $fullMonthName $currentYear");
            $lastDayOfMonth = Carbon::parse("last day of $fullMonthName $currentYear");
            // Loop through each week of the month
            for ($weekStart = $firstDayOfMonth->copy()->startOfWeek(); $weekStart->lessThanOrEqualTo($lastDayOfMonth); $weekStart->addWeek()) {
                $weekEnd = $weekStart->copy()->endOfWeek()->min($lastDayOfMonth)->endOfDay();
                // Add week label if not already added
                $weekLabel = 'Week ' . $weekStart->weekOfMonth;
                if (!in_array($weekLabel, $data['labels'])) {
                    $data['labels'][] = $weekLabel;
                }
                // Count open responses
                $openCount = $openResponses->whereBetween('created_at', [$weekStart, $weekEnd])->count();
                $clickCount = 0;
                // Count click responses
                foreach ($answers as $answer) {
                    foreach ($answer->reactCounts as $react) {
                        if (Carbon::parse($react->created_at)->between($weekStart, $weekEnd)) {
                            $clickCount++;
                        }
                    }
                }
                // Add data for the current week
                $weekDataOpen[] = $openCount;
                $weekDataClick[] = $clickCount;
            }
            // Ensure that both the open and click arrays have data for 5 weeks
            for ($i = count($weekDataOpen); $i < 5; $i++) {
                $weekDataOpen[] = 0;
                $weekDataClick[] = 0;
            }
            $data['open'][strtolower($shortMonthName)] = $weekDataOpen;
            $data['click'][strtolower($shortMonthName)] = $weekDataClick;
        }
        // Sort labels to ensure correct order
        sort($data['labels']);

        return $data;
    }
    // all polss analytics counts
    public function pollsAnalyticsCount(Request $request)
    {
        $selectedYear = $request->input('year', Carbon::now()->year);
        $currentYear = Carbon::now()->year;

        // Fetch monthly data for the selected year
        $monthlyPollData = DB::table('polls')
            ->select(DB::raw('DATE_FORMAT(created_at, "%m-%Y") as month'), DB::raw('COUNT(*) as count'))
            ->whereYear('created_at', $selectedYear)
            ->groupBy('month')
            ->orderBy('month', 'asc')
            ->get();

        // Transform monthly data
        $monthlyData = $monthlyPollData->mapWithKeys(function ($item) {
            return [$item->month => ['count' => $item->count]];
        })->toArray();

        // Fetch yearly data for a range of years (e.g., past 5 years to future 5 years)
        $yearlyPollData = DB::table('polls')
            ->select(DB::raw('YEAR(created_at) as year'), DB::raw('COUNT(*) as count'))
            ->whereBetween('created_at', [Carbon::now()->subYears(10)->startOfYear(), Carbon::now()->addYears(2)->endOfYear()])
            ->groupBy('year')
            ->orderBy('year', 'asc')
            ->get();

        // Transform yearly data
        $yearlyData = $yearlyPollData->mapWithKeys(function ($item) {
            return [$item->year => $item->count];
        })->toArray();
        // Respond based on the request type
        // return $this->calculateParticipationRate();
        if ($request->ajax()) {
            return response()->json([
                'monthlyData' => $monthlyData,
                'yearlyData' => $yearlyData,
                'avgYEarlydata' => $this->calculateParticipationRate()

            ]);
        }

        // For non-AJAX requests, include both monthly and yearly data
        return view('poll.PollsAnalytics', [
            'monthlyData' => $monthlyData,
            'yearlyData' => $yearlyData,
            'avgYEarlydata' => $this->calculateParticipationRate()
        ]);
    }
    // All polls avge by month by  year
    public function getAvgParticipationRate(Request $request)
    {
        $year = $request->input('year', null);
        return response()->json($this->calculateParticipationRate($year));
    }


    public function calculateParticipationRate($filterYear = null)
    {
        $currentYear = Carbon::now()->year;
        $startYear = $currentYear - 10;
        // Handle yearly data calculation
        if ($filterYear === null) {
            $yearlyData = [];

            for ($year = $startYear; $year <= $currentYear; $year++) {
                $totalPollsYearly = Poll::whereYear('created_at', $year)->count();
                $totalLikesYearly = Poll::join('poll_answers', 'polls.id', '=', 'poll_answers.poll_id')
                    ->leftJoin('reacts', function ($join) use ($year) {
                        $join->on('poll_answers.id', '=', 'reacts.reactable_id')
                            ->where('reacts.reactable_type', PollAnswer::class)
                            ->whereYear('reacts.created_at', $year);
                    })
                    ->count('reacts.id');

                $yearlyAvgParticipationRate = $totalPollsYearly > 0 ? $totalLikesYearly / $totalPollsYearly : 0;

                $yearlyData[$year] = [
                    'label' => $year,
                    'total_polls' => $totalPollsYearly,
                    'total_likes' => $totalLikesYearly,
                    'avg_participation_rate' => round($yearlyAvgParticipationRate, 2),
                ];
            }

            return [
                'labels' => array_keys($yearlyData),
                'data' => array_column($yearlyData, 'avg_participation_rate'),
            ];
            // Handle monthly data calculation
        } else {
            $monthlyData = [];
            for ($month = 1; $month <= 12; $month++) {
                $totalPollsMonthly = Poll::whereYear('created_at', $filterYear)
                    ->whereMonth('created_at', $month)
                    ->count();
                $totalLikesMonthly = Poll::join('poll_answers', 'polls.id', '=', 'poll_answers.poll_id')
                    ->leftJoin('reacts', function ($join) use ($filterYear, $month) {
                        $join->on('poll_answers.id', '=', 'reacts.reactable_id')
                            ->where('reacts.reactable_type', PollAnswer::class)
                            ->whereYear('reacts.created_at', $filterYear)
                            ->whereMonth('reacts.created_at', $month);
                    })
                    ->count('reacts.id');
                $monthlyAvgParticipationRate = $totalPollsMonthly > 0 ? $totalLikesMonthly / $totalPollsMonthly : 0;
                $monthlyData[$month] = [
                    'label' => Carbon::create()->month($month)->format('F'),
                    'total_polls' => $totalPollsMonthly,
                    'total_likes' => $totalLikesMonthly,
                    'avg_participation_rate' => round($monthlyAvgParticipationRate, 2),
                ];
            }

            return [
                'labels' => array_column($monthlyData, 'label'),
                'data' => array_column($monthlyData, 'avg_participation_rate'),
            ];
        }
    }
    private function getMonthName($monthNumber)
    {
        return Carbon::create()->month($monthNumber)->format('F');
    }
}
