<?php

namespace App\Http\Controllers;

use App\Events\SendAction;
use App\Models\Suggestion;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\URL;
use Yajra\DataTables\Facades\DataTables;

class SuggestionController extends Controller
{
    public function getSuggestions(Request $request)
    {
        if ($request->ajax()) {
            $data = Suggestion::where('company_id', auth()->user()->company_id)
                ->with(['user' => function ($q) {
                    $q->select('id', 'user_name', 'department_id');
                }])
                ->withCount('dislikes', 'likes')
                ->orderBy('created_at', 'desc');

            if ($request->tab === 'reject') {
                $data->where('status', 'reject');
            } elseif ($request->tab === 'approve') {
                $data->where('status', 'approve');
            } else {
                $data->where('status', 'pending');
            }
            // } else {
            //     $data->where(function ($query) {
            //         $query->where('is_competition', false)
            //             ->where('is_announcement', false);
            //     })->orWhere(function ($query) {
            //         $query->where('is_announcement', true)
            //             ->where('created_at', '<', Carbon::now()->subDay())
            //             ->where('is_competition', false);
            //     });
            // }

            $sugesstions = $data->get();

            return DataTables::of($sugesstions)
                ->addIndexColumn()
                ->addColumn('action', function ($suggestion) {
                    return '<button data-bs-toggle="modal" data-bs-target="#responsive-modal" title="Reject Bright Ideas" onclick="populateModal(' . $suggestion->id . ')" class="btn btn-danger rounded-circle"><i class="fa fa-times"></i></button>' .
                        '<button title="Approve Bright Ideas" onclick="approveSuggestion(' . $suggestion->id . ')" class="btn btn-success rounded-circle ml-1"><i class="fa fa-check"></i></button>';
                })

                ->addColumn('dislikes_count', function ($sugestion) {
                    return '<i class="fas fa-thumbs-down"></i> ' . ($sugestion->dislikes_count ?? 0);
                })
                ->addColumn('likes_count', function ($sugestion) {
                    return '<i class="fa fa-thumbs-up"></i> ' . ($sugestion->likes_count ?? 0);
                })
                ->addColumn('media', function ($sugestion) {
                    $mediaUrl = $sugestion->media != null ? env('MEDIA_URL') . $sugestion->media : 'images/placeholderprofile.png';
                    if ($sugestion->media_type == 'video') {
                        return '<a href="' . URL::asset($mediaUrl) . '" target="_blank"><video src="' . URL::asset($mediaUrl) . '" controls width="100">Your browser does not support the video tag.</video></a>';
                    } else {
                        return '<a href="' . URL::asset($mediaUrl) . '" target="_blank"><img src="' . URL::asset($mediaUrl) . '" class="img-circle" style="height: 40px; width: 40px;" /></a>';
                    }
                })
                ->addColumn('created_at', function ($sugestion) {
                    return $sugestion->created_at->format('Y-m-d');
                })
                ->rawColumns(['action', 'dislikes_count', 'likes_count', 'media'])
                ->make(true);
        }

        return view('suggestion.SuggestionList');
    }

    function barchartIndex(Request $request)
    {
        $users = User::where(['company_id' => auth()->user()->company_id, 'user_role' => 'user'])->count();
        $currentYear = Carbon::now()->year;
        if ($users == 0) {
            return redirect()->back()->with('error', 'You cannot see the data because there are no users in the app.');
        }
        if ($request->ajax()) {
            $data = Suggestion::where('company_id', auth()->user()->company_id)
                ->whereYear('created_at', $currentYear)
                ->with(['user' => function ($q) {
                    $q->select('id', 'user_name', 'department_id');
                }])
                ->withCount('dislikes', 'likes')
                ->orderBy('created_at', 'desc')
                ->where('status', 'approve');

            $suggestions = $data->get();

            return DataTables::of($suggestions)
                ->addColumn('dislikes_count', function ($suggestion) {
                    return '<i class="fas fa-thumbs-down"></i> ' . ($suggestion->dislikes_count ?? 0);
                })
                ->addColumn('dislikes', function ($suggestion) {
                    return ($suggestion->dislikes_count ?? 0);
                })
                ->addColumn('likes', function ($suggestion) {
                    return ($suggestion->likes_count ?? 0);
                })
                ->addColumn('likes_count', function ($suggestion) {
                    return '<i class="fa fa-thumbs-up"></i> ' . ($suggestion->likes_count ?? 0);
                })
                ->addColumn('media', function ($sugestion) {
                    $mediaUrl = $sugestion->media != null ? env('MEDIA_URL') . $sugestion->media : 'images/placeholderprofile.png';
                    if ($sugestion->media_type == 'video') {
                        return '<a href="' . URL::asset($mediaUrl) . '" target="_blank"><video src="' . URL::asset($mediaUrl) . '" controls width="100">Your browser does not support the video tag.</video></a>';
                    } else {
                        return '<a href="' . URL::asset($mediaUrl) . '" target="_blank"><img src="' . URL::asset($mediaUrl) . '" class="img-circle" style="height: 50px; width: 50px;" /></a>';
                    }
                })
                ->addColumn('created_at', function ($suggestion) {
                    return $suggestion->created_at->format('Y-M-d');
                })
                ->rawColumns(['dislikes_count', 'likes_count', 'media'])
                ->make(true);
        }
        return view('suggestion.SugesstionAnalytics', compact('users'));
    }

    function statusUpdate(Request $request, $suggestion_id = "", $status = "")
    {
        // return $request;
        $id = $request->reason ? $request->suggestionId : $suggestion_id;
        $suggestion = Suggestion::find($id);

        $updateData = ['status' => $request->status];
        if ($request->reason) {
            $updateData['reason'] = $request->reason;
        }
        Suggestion::where('id', $id)->update($updateData);
        $actionTitle = $request->reason ? 'Your Bright Ideas is rejected by an admin' : 'Your Bright Ideas is approved by an admin';
        $actionBody = $request->reason ? $request->reason : 'Bright Ideas approved by admin';
        event(new SendAction(
            [$suggestion->user_id],
            [
                'title' => $actionTitle,
                'body' => $actionBody,
                'object_id' => $suggestion->id,
                'object_type' => get_class($suggestion),
                'actor_id' => auth()->user()->id,
                'type' => 'suggestion_status',
                'object' => 'suggestion',
                'actor_type' => auth()->user()->user_role,
                'user_id' => $suggestion->user_id
            ],
            true
        ));
        $message = $request->reason ? 'Bright Ideas rejected successfully' : 'Bright Ideas approved successfully';
        return redirect()->back()->with('message', $message);
    }
}
