<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Models\Follow;
use App\Events\SendAction;
use App\Models\Block;
use App\Models\BlockedUser;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;

class UserController extends Controller
{
    function getUsers(Request $request)
    {

        if ($request->ajax()) {
            if (auth()->user()->user_role == 'sub_admin') {
                $users = User::where('user_role', 'user')
                    ->where('company_id', auth()->user()
                        ->company_id)->orderBy('created_at', 'desc');
            } elseif (auth()->user()->user_role == 'admin') {
                $users = User::where('user_role', 'sub_admin')
                    ->where('company_id', auth()->user()
                        ->company_id)->orderBy('created_at', 'desc');
            }

            return DataTables::of($users)
                ->addIndexColumn()
                ->addColumn('action', function ($user) {
                    $viewUrl = route('user.info', $user->id);
                    $blockUnblockUrl = route('blockAndUnblock.user', ['user_id' => $user->id, 'status' => $user->status == 1 ? 0 : 1]);
                    $blockUnblockIcon = $user->status == 1 ? 'fa-lock' : 'fa-lock-open';
                    $blockUnblockColor = $user->status == 1 ? 'rgb(248, 52, 52)' : 'rgb(109, 214, 39)';

                    return '<a href="' . $viewUrl . '"><i title="User Info" class="far fa-eye" style="font-size:20px"></i></a>'
                        . '<span style="margin-left: 10%;"></span>';
                    // return '<a href="' . $viewUrl . '"><i title="User Info" class="far fa-eye" style="font-size:20px"></i></a>'
                    //     . '<span style="margin-left: 10%;"><a href="#" onclick="swalll(\'' . $blockUnblockUrl . '\', \'Block/Unblock User\', \'Are you sure you want to ' . ($user->status == 1 ? 'unblock' : 'block') . ' this user?\', \'User ' . ($user->status == 1 ? 'unblocked' : 'blocked') . ' successfully\')">'
                    //     . '<i title="' . ($user->status == 1 ? 'Unblock' : 'Block') . ' User" class="fa ' . $blockUnblockIcon . '" style="font-size:20px;color: ' . $blockUnblockColor . '"></i></a></span>';
                })
                ->rawColumns(['action'])
                ->make(true);
        }

        return view("Users.UserList");
    }


    function userinfo($user)
    {
        $user = User::where('id', $user)->with([
            'department',
            'incident',
            'suggestions' => function ($query) {
                $query->withCount('likes', 'dislikes');
            }
        ])->first();

        // Check if user is found
        if ($user) {
            // Reset appends for incident
            $user->incident->each->setAppends([]);

            // Filter suggestions based on status
            $approvedSuggestions = $user->suggestions->where('status', 'approve');
            $pendingSuggestions = $user->suggestions->where('status', 'pending');
            $rejectedSuggestions = $user->suggestions->where('status', 'reject');

            // Reset appends for each suggestion
            $approvedSuggestions->each->setAppends([]);
            $pendingSuggestions->each->setAppends([]);
            $rejectedSuggestions->each->setAppends([]);

            // Assign filtered suggestions to user object
            $user->approvedSuggestions = $approvedSuggestions;
            $user->pendingSuggestions = $pendingSuggestions;
            $user->rejectedSuggestions = $rejectedSuggestions;
        }
        return view('Users.userInfo', compact('user'));
    }




    /**
     * editUser
     *
     * @param  mixed $user
     * @return void
     */
    function editUser(User $user)
    {
        return view('Users.UserUpdateForm', compact('user'));
    }

    /**
     * updateuser
     *
     * @param  mixed $request
     * @param  mixed $user
     * @return void
     */
    function updateuser(Request $request, $user)
    {

        $this->validate($request, [
            'water_goal' => 'required',
            'user_type' => "required",
            'user_role' => "required"
        ]);
        $data = $request->only([
            'water_goal',
            'user_type',
            'user_role'
        ]);
        User::where('id', $user)->update($data);
        return redirect(Route('user.index'))->with('message', "User Updated Successfully");
    }

    /**
     * deleteuser
     *
     * @param  mixed $user
     * @return void
     */
    function deleteuser(User $user)
    {
        $user->delete();
        return redirect(Route('user.index'))->with('message', "User Deleted Successfully");
    }

    function blockAndUnblock(User $user_id, $status = 0)
    {
        // return $user_id;
        $user_id->update(['status' => $status]);
        if ($status == 1) {

            return redirect(Route('user.index'))->with('message', "User unblocked Successfully");
        } else {
            $userID = $user_id->id;
            event(new SendAction(
                [$userID],
                [
                    'title' => 'Your Account is block by an admin',
                    'body' => 'Account is block by admin',
                    'object_id' => $userID,
                    'object_type' => 'App\Models\User',
                    'actor_id' => auth()->user()->id,
                    'type' => 'blocked',
                    'object' => 'user',
                    'actor_type' => 'admin',
                    'user_id' => $userID
                ],
                false
            ));
            return redirect(Route('user.index'))->with('message', "User blocked Successfully");
        }
    }

    function BlockedUserList()
    {
        $block_users = Block::with('user', 'blockedUser')->get();
        return view('Users.BlockUserList', compact('block_users'));
    }
}
