<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;
use Stripe\StripeClient;
use App\Models\Department;
use Laravel\Sanctum\HasApiTokens;
use Spatie\Permission\Models\Role;
use Illuminate\Support\Facades\Hash;
use Illuminate\Notifications\Notifiable;
use Spatie\Permission\Models\Permission;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Spatie\Permission\Traits\HasRoles;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable, SoftDeletes, HasRoles;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'company_id',
        'department_id',
        'forgot_token',
        'first_name',
        'last_name',
        'user_name',
        'email',
        'verification_code',
        'forgot_pasword_secret_key',
        'expired_at',
        'email_verified_at',
        'social_id',
        'provider',
        'city',
        'state',
        'phone',
        'phone_verification_code',
        'phone_verified_at',
        'password',
        'forgot_password_code',
        'gender',
        'dob',
        'profile_image',
        'customer_stripe_id',
        'user_type',
        'user_role',
        'remember_token',
        'status'
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    protected $append = [
        'profile_image_url'
    ];

    public function department()
    {
        return $this->belongsTo(Department::class, 'department_id');
    }

    public function rolesCustomRelation()
    {
        return $this->belongsToMany(Role::class, 'model_has_roles', 'model_id', 'role_id')->limit(1);
    }




    public function getProfileImageUrlAttribute()
    {
        return $this->profile_image ? env('MEDIA_URL') . $this->profile_image : null;
    }

    // public function setProfileImageAttribute($profile_image)
    // {
    //     if ($this->profile_image && file_exists(env('DISK') . $this->profile_image)) {
    //         unlink(env('DISK') . $this->profile_image);
    //     }

    //     if (is_file($profile_image)) {
    //         $this->attributes['profile_image'] = $profile_image->store("users/profile-image", env('DISK'));
    //     }
    // }

    // public function setPasswordAttribute($password)
    // {
    //     $this->attributes['password'] = Hash::make($password);
    // }

    public function devices()
    {
        return $this->hasMany(UserDevice::class, 'user_id');
    }


    /**
     * Get all of the posts for the User
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function posts()
    {
        return $this->hasMany(Post::class);
    }


    public function userDevices($request, $access_token)
    {
        $request['access_token'] = $access_token;
        $this->devices()->updateOrCreate(
            $request->only('udid'),
            $request->all()
        );
    }

    public function cards()
    {
        return $this->hasMany(UserCard::class, 'user_id');
    }

    public function charge($card, $data)
    {
        $stripe = new StripeClient(env('STRIPE_SK', 'sk_test_yo96XLBt4O0xQZPCteMVlW3a'));
        $intent = $stripe->paymentIntents->create([
            "amount" => $data['amount'] * 100, // amount in cents
            "currency" => 'GBP',
            "payment_method_types" => ['card'],
            "payment_method_options" => ['card' => ['setup_future_usage' => 'off_session']],
            "description" => $data['description'],
            "customer" => $this->customer_stripe_id,
            "metadata" => $data['metadata'],
        ])->confirm([
            'payment_method' => $card,
            'return_url' => $data['return_url'],
        ]);

        return $intent;
    }


    public function suggestions()
    {
        return $this->hasMany(Suggestion::class, 'user_id');
    }
    public function incident()
    {
        return $this->hasMany(Incident::class, 'user_id');
    }


    public function company()
    {
        return $this->belongsTo(Company::class, 'company_id');
    }
}
