<?php

namespace App\Traits;

use Carbon\Carbon;
use App\Models\Poll;
use App\Models\User;
use App\Models\Department;
use App\Models\PollAnswer;
use App\Models\React;
use Illuminate\Support\Facades\DB;


trait CustomAnalytics
{
    public function CustomLinchart($id)
    {
        $currentYear = Carbon::now()->year;
        $polls = Poll::where('id', $id)
            ->with([
                'answers' => function ($q) use ($currentYear) {
                    $q->select('poll_answers.*', DB::raw('MONTH(reacts.created_at) as month'))
                        ->join('reacts', 'poll_answers.id', '=', 'reacts.reactable_id')
                        ->where('reacts.reactable_type', 'App\Models\PollAnswer')
                        ->whereYear('reacts.created_at', $currentYear)
                        ->addSelect(DB::raw('COUNT(reacts.id) as reacts_count')) // Count reacts for each poll answer per month
                        ->groupBy('poll_answers.id', 'poll_answers.poll_id', 'poll_answers.answer', 'poll_answers.emoji', 'poll_answers.color', 'poll_answers.emoji_id', 'poll_answers.grade', 'poll_answers.created_at', 'poll_answers.updated_at', 'poll_answers.deleted_at', DB::raw('MONTH(reacts.created_at)'));
                }
            ])
            ->first();
        $totalReactCountByMonth = [];
        // Initialize totalReactCountByMonth with all months and default values
        foreach (range(1, 12) as $month) {
            $monthName = $this->getMonthName($month);
            $totalReactCountByMonth[$monthName] = [
                'promoter' => 0,
                'passive' => 0,
                'detractor' => 0,
            ];
        }
        foreach ($polls->answers as $answer) {
            // dd($answer->reacts);
            $monthName = $this->getMonthName($answer->month);

            $category = '';
            if ($answer->grade == 1 || $answer->grade == 2) {
                $category = 'promoter';
            } elseif ($answer->grade == 3) {
                $category = 'passive';
            } elseif ($answer->grade == 4 || $answer->grade == 5) {
                $category = 'detractor';
            }

            $totalReactCountByMonth[$monthName][$category] += $answer->reacts_count;
        }
        return $totalReactCountByMonth;
    }

    public function customPieChart($total_company_users, $id, $companyId)
    {
        if ($total_company_users == 0) {
            return redirect()->back()->with('error', 'Total company users cannot be zero');
        }
        $poll = Poll::where('company_id', $companyId)
            ->where('id', $id)
            ->with([
                'answers' => function ($q) {
                    $q->select('poll_answers.*', DB::raw('MONTH(created_at) as month'))
                        ->withCount('reactCounts');
                }
            ])
            ->first();
        $department_ids = Department::where('company_id', $companyId)->pluck('id');
        $usersByDepartment = User::whereIn('department_id', $department_ids)
            ->whereNotNull('department_id')
            ->get()
            ->groupBy('department_id');
        $departments = [];
        $departmentReactCounts = [];
        $colors = [];
        foreach ($usersByDepartment as $departmentId => $users) {
            $totalReactCount = 0;
            $department = Department::find($departmentId);
            if ($department) {
                $departments[] = $department->name;

                // Calculate the total react count for the department
                foreach ($users as $user) {
                    foreach ($poll->answers as $answer) {
                        if ($answer->reactCounts->where('user_id', $user->id)->count()) {
                            $totalReactCount += $answer->reactCounts->where('user_id', $user->id)->count();
                        }
                    }
                }

                $departmentReactCounts[] = $totalReactCount;
                $colors[] = sprintf('#%06X', mt_rand(0, 0xFFFFFF)); // Random color
            }
        }
        $colors[] = '#CCCCCC';
        return [
            'departments' => $departments,
            'departmentReactCounts' => $departmentReactCounts,
            'colors' => $colors,
            'TotalPercentage' => array_sum($departmentReactCounts) / $total_company_users * 100,
        ];
    }
    public function customBArChart($total_company_users, $id, $companyId)
    {
        if ($total_company_users == 0) {
            return redirect()->back()->with('error', 'Total company users cannot be zero');
        }
        $pollAnswers = Poll::where('company_id', $companyId)
            ->where('id', $id)
            ->with([
                'answers' => function ($q) {
                    $q->withCount('reactCounts');
                }
            ])
            ->first();
        $pollAnswers['totalUsers'] = $total_company_users;
        return $pollAnswers;
    }

    public function CommunicationsReachEngagement($total_company_users, $id, $companyId)
    {
        $poll_engagement = $polls = Poll::where('id', $id)->with([
            'answers' => function ($q) {
                $q->withCount('reactCounts');
            }
        ])->first();
        return $totalReactCounts = $poll_engagement->answers->sum('react_counts_count') / $total_company_users * 100;
    }
    public function clickThroughRateCount($id)
    {
        $total = Poll::where('id', $id)->withCount('clickThroughRate')->first();
        $totalOpen = $total->click_through_rate_count;
        if ($totalOpen == 0) {
            return 0;
        }
        $poll_click = Poll::where('id', $id)->with([
            'answers' => function ($q) {
                $q->withCount('reactCounts');
            }
        ])->first();
        $totalReactCounts = $poll_click->answers->sum('react_counts_count');
        return ($totalReactCounts / $totalOpen) * 100;
    }


    public function PeakTimesClickMaps($total_company_users, $id, $companyId)
    {
        $poll = Poll::with(['clickThroughRate', 'answers.reactCounts'])->findOrFail($id);
        $totalOpen = $poll->clickThroughRate->count(); // Assuming clickThroughRate is a relationship
        $answers = $poll->answers;
        // Retrieve open responses (clickThroughRate data)
        $openResponses = $poll->clickThroughRate;
        // Process the data into time intervals
        $PeakTimesClickMaps = $this->processData($answers, $openResponses, $totalOpen);
        return $PeakTimesClickMaps;
    }
    // peak time mape datasets
    private function processData($answers, $openResponses, $totalOpen)
    {
        $data = [
            '1hr' => ['labels' => [], 'open' => [], 'click' => []],
            '3hr' => ['labels' => [], 'open' => [], 'click' => []],
            '6hr' => ['labels' => [], 'open' => [], 'click' => []],
            'full' => ['labels' => [], 'open' => [], 'click' => []],
            'currentMonth' => ['labels' => [], 'open' => [], 'click' => []]
        ];

        // Define intervals
        $timeIntervals = [
            '1hr' => [
                'duration' => 1,
                'step' => 15
            ],
            '3hr' => [
                'duration' => 3,
                'step' => 30
            ],
            '6hr' => [
                'duration' => 6,
                'step' => 60
            ],
            'full' => [
                'duration' => 24,
                'step' => 60
            ]
        ];

        // Calculate start and end of current month
        $startOfMonth = Carbon::now()->startOfMonth();
        $endOfMonth = Carbon::now()->endOfMonth();
        foreach ($timeIntervals as $interval => $details) {
            $hours = $details['duration'];
            $stepMinutes = $details['step'];

            for ($i = 0; $i <= $hours * 60; $i += $stepMinutes) {
                $label = sprintf('%02d:%02d', floor($i / 60), $i % 60);
                $data[$interval]['labels'][] = $label;

                $start = Carbon::now()->startOfDay()->addMinutes($i);
                $end = Carbon::now()->startOfDay()->addMinutes($i + $stepMinutes);

                $openCount = $openResponses->whereBetween('created_at', [$start, $end])->count();
                $clickCount = 0;

                foreach ($answers as $answer) {
                    foreach ($answer->reactCounts as $react) {
                        if (Carbon::parse($react->created_at)->between($start, $end)) {
                            $clickCount++;
                        }
                    }
                }

                $data[$interval]['open'][] = $openCount;
                $data[$interval]['click'][] = $clickCount;
            }
        }
        // Process data for current month
        for ($day = $startOfMonth; $day <= $endOfMonth; $day->addDay()) {
            $label = $day->format('Y-m-d');
            $data['currentMonth']['labels'][] = $label;
            $fullStart = $day->format('Y-m-d') . 'T00:00:00.000000Z';
            $fullEnd = $day->format('Y-m-d') . 'T23:59:59.999999Z';
            $openCount = $openResponses->whereBetween('created_at', [Carbon::parse($fullStart), Carbon::parse($fullEnd)])->count();
            $clickCount = 0;
            foreach ($answers as $answer) {
                foreach ($answer->reactCounts as $react) {
                    if (Carbon::parse($react->created_at)->between($fullStart, $fullEnd)) {
                        $clickCount++;
                    }
                }
            }
            $data['currentMonth']['open'][] = $openCount;
            $data['currentMonth']['click'][] = $clickCount;
        }
        return $data;
    }
    private function dataOverviewOpenRate($id)
    {
        $poll_engagement = Poll::where('id', $id)->with([
            'answers' => function ($q) {
                $q->withCount('reactCounts');
            }
        ])->first();
        $total_all_company_users = User::where('user_role', 'user')->count();
        if ($total_all_company_users === 0) {
        }
        $totalReactCounts = ($poll_engagement->answers->sum('react_counts_count') / $total_all_company_users) * 100;

        return $totalReactCounts;
    }

    public function pollsCountBymonthByyear($request)
    {
        $selectedYear = $request->input('year', Carbon::now()->year);

        $pollData = DB::table('polls')
            ->select(DB::raw('DATE_FORMAT(created_at, "%m-%Y") as month'), DB::raw('COUNT(*) as count'))
            ->whereYear('created_at', $selectedYear)
            ->groupBy('month')
            ->orderBy('month', 'asc')
            ->get();

        return $pollData->mapWithKeys(function ($item) {
            return [$item->month => ['count' => [$item->count]]];
        })->toArray();
    }
}
