@extends('layouts.master')
@section('contain')
    <!-- Page titles -->
    <div class="row page-titles">
        <!-- Title based on whether you're updating or adding a department -->
        <div class="col-md-5 align-self-center">
            @if (isset($department))
                <h4 class="card-title">Update Department</h4>
            @else
                <h4 class="card-title">Add Department</h4>
            @endif
        </div>
        <div class="col-md-7 align-self-center text-end">
            <!-- Breadcrumbs -->
            <div class="d-flex justify-content-end align-items-center">
                <ol class="breadcrumb justify-content-end">
                    <li class="breadcrumb-item active"><a class="text-cyan" href="{{ url('/') }}">Home</a></li>
                    <li class="breadcrumb-item active"><a class="text-cyan" href="{{ Route('department.index') }}">Departments</a>
                    </li>
                </ol>
            </div>
        </div>
    </div>
    <!-- End Page titles -->

    <!-- news_feed Form -->
    <div class="card">
        <div class="card-body">
            <h4 class="card-title"></h4>
            @if (isset($department))
                <!-- Form for updating an existing department -->
                <form method="POST" action={{ Route('department.update', $department->id) }} enctype="multipart/form-data"
                    data-parsley-validate>
                    @csrf
                    @method('PUT')
                @else
                    <!-- Form for adding a new department -->
                    <form action="{{ Route('department.add') }}" method="POST" enctype="multipart/form-data"
                        data-parsley-validate>
                        @csrf
            @endif
            <div class="row">
                <div class="form-group col-md-6 ">
                    <label class="font-bold">Name*</label>
                    <!-- Input for the description -->
                    <input required data-parsley-required-message="name field is required"
                        data-parsley-maxlength="150"
                        data-parsley-maxlength-message="name must not exceed 150 characters" type="text"
                        name="name" value="{{ $department->name ?? old('name') }}"
                        class="form-control @error('name') is-invalid @enderror" placeholder="Enter name">
                    @error('name')
                        <span class="invalid-feedback" role="alert">
                            <strong>{{ $message }}</strong>
                        </span>
                    @enderror
                </div>

            </div>
            <div class="form-group col-md-12">
                <!-- Submit button -->
                @if (isset($department))
                    <button type="submit" class="btn btn-info me-2 text-white">Update</button>
                @else
                    <button type="submit" class="btn btn-success me-2 text-white">Submit</button>
                @endif
            </div>
            </form>
        </div>
    </div>



    <style>
        .img-div {
            position: relative;
            width: 10%;
            float: left;
            margin-right: 5px;
            margin-left: 5px;
            margin-bottom: 10px;
            margin-top: 10px;
        }

        .image {
            width: 100px;
            height: 100px;
            object-fit: cover;
        }

        .middle {
            transition: .5s ease;
            opacity: 0;
            position: absolute;
            top: 50%;
            left: 50%;
            transform: translate(-50%, -50%);
            -ms-transform: translate(-50%, -50%);
            text-align: center;
        }

        .img-div:hover .image {
            opacity: 0.3;
        }

        .img-div:hover .middle {
            opacity: 1;
        }
    </style>
    <script>
        $(document).ready(function() {
            // check unique title


            // update image remove and set ids
            $('.img-div').hover(function() {
                $(this).find('.delete-btn').css('opacity', '1');
                $(this).find('img').css('opacity', '0.3');
            }, function() {
                $(this).find('.delete-btn').css('opacity', '0');
                $(this).find('img').css('opacity', '1');
            });

            $('.delete-btn').click(function() {
                var imageId = $(this).data('id');
                $(this).closest('.img-div').remove();
                var deletedImageIds = $('#deleted_image_ids').val().split(',').filter(Boolean);
                deletedImageIds.push(imageId);
                $('#deleted_image_ids').val(deletedImageIds.length > 0 ? deletedImageIds.join(',') : '');
            });
        });
        // update image remove and set ids end

        // create images strat
        $(document).ready(function() {
            // var fileArr = [];
            // $("#images").change(function() {
            //     // check if fileArr length is greater than 0
            //     if (fileArr.length > 0) fileArr = [];

            //     $('#image_preview').html("");
            //     var total_file = document.getElementById("images").files;
            //     if (!total_file.length) return;
            //     for (var i = 0; i < total_file.length; i++) {
            //         var fileType = total_file[i].type.split('/')[0];
            //         if (total_file[i].size > 1048576) {
            //             return false;
            //         } else {
            //             fileArr.push(total_file[i]);
            //             $('#image_preview').append("<div class='img-div' id='img-div" + i + "'><img src='" +
            //                 URL.createObjectURL(event.target.files[i]) +
            //                 "' class='img-responsive image img-thumbnail' title='" + total_file[i]
            //                 .name + "'><div class='middle'><button id='action-icon' value='img-div" +
            //                 i + "' class='btn btn-danger' role='" + total_file[i].name +
            //                 "'><i class='fa fa-trash'></i></button></div></div>");
            //         }
            //     }

            // });
            var fileArr = [];
            $("#images").change(function(event) {
                // Clear fileArr if it's not empty
                if (fileArr.length > 0) fileArr = [];

                $('#image_preview').html("");
                var total_files = event.target.files;
                if (!total_files.length) return;

                for (var i = 0; i < total_files.length; i++) {
                    var fileType = total_files[i].type.split('/')[0];
                    if (total_files[i].size > 10485765111) {
                        // File size exceeds the limit
                        return false;
                    } else {
                        fileArr.push(total_files[i]);
                        console.log(fileType);
                        if (fileType === 'image') {
                            // Display image tag for image files
                            $('#image_preview').append("<div class='img-div' id='img-div" + i +
                                "'><img src='" +
                                URL.createObjectURL(total_files[i]) +
                                "' class='img-responsive image img-thumbnail' title='" + total_files[i]
                                .name +
                                "'><div class='middle'><button id='action-icon' value='img-div" +
                                i + "' class='btn btn-danger' role='" + total_files[i].name +
                                "'><i class='fa fa-trash'></i></button></div></div>");
                        } else if (fileType === 'video') {
                            // Display video tag for video files
                            $('#image_preview').append("<div class='img-div' id='img-div" + i +
                                "'><video class='img-responsive image img-thumbnail' controls><source src='" +
                                URL.createObjectURL(total_files[i]) +
                                "' type='video/mp4'>Your browser does not support the video tag.</video><div class='middle'><button id='action-icon' value='img-div" +
                                i + "' class='btn btn-danger' role='" + total_files[i].name +
                                "'><i class='fa fa-trash'></i></button></div></div>");
                        }
                    }
                }
            });


            // create images strat end
            $('body').on('click', '#action-icon', function(evt) {
                var divName = this.value;
                var fileName = $(this).attr('role');
                $(`#${divName}`).remove();

                for (var i = 0; i < fileArr.length; i++) {
                    if (fileArr[i].name === fileName) {
                        fileArr.splice(i, 1);
                    }
                }
                document.getElementById('images').files = FileListItem(fileArr);
                evt.preventDefault();
            });

            function FileListItem(file) {
                file = [].slice.call(Array.isArray(file) ? file : arguments)
                for (var c, b = c = file.length, d = !0; b-- && d;) d = file[b] instanceof File
                if (!d) throw new TypeError("expected argument to FileList is File or array of File objects")
                for (b = (new ClipboardEvent("")).clipboardData || new DataTransfer; c--;) b.items.add(file[c])
                return b.files
            }
        });
        $(document).ready(function() {
            var exampleUrl = "http://example.com";
            var errorMessage = "this field is required and  enter a valid HTTP URL. Example: " + exampleUrl;
            $('#exampleInputEmail111').parsley().on('field:error', function() {
                var errorElement = $(this.$element).closest('.form-group').find(
                    '.parsley-errors-list.filled');
                errorElement.html(errorMessage);
            });
        });
    </script>
    <script>
        var numberMask = IMask(
            document.getElementById('number-mask'), {
                mask: Number,
                min: -10000,
                max: 10000,
                thousandsSeparator: ' '
            });

        function swalll(id) {
            event.preventDefault();
            swal({
                    title: "Are you sure?",
                    text: "Once deleted, you will not be able to recover this imaginary field !",
                    icon: "warning",
                    buttons: true,
                    dangerMode: true,
                })
                .then((willDelete) => {
                    if (willDelete) {
                        var formId = "delete-form-" + id;
                        document.getElementById(formId).submit();
                        // swal("Poof! Your imaginary file has been deleted!", {
                        //     icon: "success",
                        // });
                    } else {
                        // swal("Your imaginary file is safe!");
                    }
                });


            // alert(id);
        }
    </script>
@endsection
