@extends('layouts.master')
@section('contain')
    <div class="row page-titles">
        <div class="col-md-5 align-self-center">
            <h3 class="text-themecolor">Good Neighbour Summaries</h3>
        </div>
        <div class="col-md-7 align-self-center text-end">
            <div class="d-flex justify-content-end align-items-center">
                <ol class="breadcrumb justify-content-end">
                    <li><a class="breadcrumb-item active" href="{{ url('/') }}">Home</a></li>
                    <li><a class="breadcrumb-item active" href="javascript:history.go(-1);">back</a></li>
                </ol>
            </div>
        </div>
    </div>
    <div class="card">
        <div class="card-body">
            <div class="container-fluid">
                <div class="row">
                    <div class="col-md-12">
                        <div class="card col-md-9" style="position: relative;">
                            <canvas id="incidentChart"></canvas>
                        </div>
                        <div class="col-md-3" id="dropdown-container" style="position: absolute; top: 0; right: 0;">
                            <div class="d-flex flex-column">
                                <div class="mb-2">
                                    <select style="font-size: smaller;" id="month-filter" class="custom-select">
                                        <option value="">Select Month to View Daily Data</option>
                                        <option value="1">January</option>
                                        <option value="2">February</option>
                                        <option value="3">March</option>
                                        <option value="4">April</option>
                                        <option value="5">May</option>
                                        <option value="6">June</option>
                                        <option value="7">July</option>
                                        <option value="8">August</option>
                                        <option value="9">September</option>
                                        <option value="10">October</option>
                                        <option value="11">November</option>
                                        <option value="12">December</option>
                                    </select>
                                </div>
                                <div class="mb-2">
                                    <select id="department-filter" class="custom-select">
                                        <option value="">Filter By Department</option>
                                        @foreach ($departments as $department)
                                            <option value="{{ $department->id }}">{{ $department->name }}</option>
                                        @endforeach
                                    </select>
                                </div>
                                <div class="card">
                                    <div class="card-body">
                                        <h5 class="font-bold">Mean time to repair</h5>
                                        <p class="card-text" style="font-size: 48px; font-weight: bold;">{{ round($avgHoursToRepair->avg_hours_to_repair, 1) }} Hrs</p>
                                    </div>
                                </div>

                            </div>
                        </div>

                    </div>
                </div>
            </div>
        </div>
    </div>

    <script>
        const ctx = document.getElementById('incidentChart').getContext('2d');
        let data = @json($formattedData);

        const statuses = ['submitted', 'received', 'on the case', 'in progress', 'resolved'];
        let chart;

        function initializeChart(data) {
            const labels = Object.keys(data);
            const datasets = statuses.map(status => {
                return {
                    label: status,
                    data: labels.map(label => data[label]['statuses'][status] || 0),
                    backgroundColor: getStatusColor(status),
                    stack: 'Stack 1'
                };
            });

            if (chart) {
                chart.destroy();
            }

            chart = new Chart(ctx, {
                type: 'bar',
                data: {
                    labels: labels,
                    datasets: datasets
                },
                options: {
                    responsive: true,
                    scales: {
                        x: {
                            stacked: true
                        },
                        y: {
                            stacked: true
                        }
                    }
                }
            });
        }

        function getStatusColor(status) {
            switch (status) {
                case 'submitted':
                    return '#f01e1e';
                case 'received':
                    return '#ffd65b';
                case 'on the case':
                    return '#ffbf00';
                case 'in progress':
                    return '#4da64d';
                case 'resolved':
                    return '#10cc10';
                default:
                    return '#201e1e';
            }
        }

        document.getElementById('month-filter').addEventListener('change', function() {
            fetchDataAndUpdateChart();
        });

        document.getElementById('department-filter').addEventListener('change', function() {
            fetchDataAndUpdateChart();
        });

        function fetchDataAndUpdateChart() {
            const selectedMonth = document.getElementById('month-filter').value;
            const selectedDepartment = document.getElementById('department-filter').value;
            const url = "{{ route('incident.summary.ajax') }}";

            $.ajax({
                url: url,
                type: 'GET',
                data: {
                    month: selectedMonth,
                    department_id: selectedDepartment
                },
                success: function(response) {
                    data = response;
                    initializeChart(data);
                }
            });
        }

        // Initialize chart with default data
        initializeChart(data);
    </script>
@endsection
